#!/usr/bin/python

# This is an example of using dynamic trees (trees where nodes are only
# calculated as they are needed) with pygtk.  This particular example
# allows for the browsing of the variables in a module, allowing the
# descent into classes and modules and other objects with a __dict__
# attribute.

# If this file is run straight, it will let you browse the Gtkinter module.

from Gtkinter import *

class BrowseTreeItem(GtkTreeItem):
	def __init__(self, name, dict=None, disp=None):
		GtkTreeItem.__init__(self, name)
		self.name = name
		self.dict = dict
		self.disp = disp
		self.exp_id = self.connect("expand", self.sig_expand)
	def init_subtree(self):
		if self.dict:
			self.subtree = GtkTree()
			self.subtree.set_selection_mode(SELECTION_BROWSE)
			self.subtree.connect("select_child", self.subsel_child)
			self.set_subtree(self.subtree)
			self.subtree.show()
	def subsel_child(self, _t, _c):
		if self.disp:
			key = _c.children()[0].get()
			if key == '__builtins__':  # builtins includes itself
				value = key
			else:
				value = `self.dict[key]`
			self.disp.set_text(value)
	def sig_select(self):
		if self.disp:
			self.disp.set_text(value)
	def sig_expand(self, _t):
		keys = self.dict.keys()
		keys.sort()
		for key in keys:
			dict = None
			try:
				dict = vars(self.dict[key])
			except TypeError:
				pass
			item = BrowseTreeItem(key, dict, self.disp)
			self.subtree.append(item)
			item.init_subtree()
			item.show()
		self.disconnect(self.exp_id)

class BrowseVariables(GtkVBox):
	def __init__(self, name, dict):
		GtkVBox.__init__(self)
		self.set_spacing(2)

		self.sw = GtkScrolledWindow()
		self.sw.set_usize(300, 200)
		self.sw.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.pack_start(self.sw)
		self.sw.show()

		self.disp = GtkEntry()
		self.disp.set_editable(FALSE)
		self.pack_start(self.disp, expand=FALSE)
		self.disp.show()

		self.root_tree = GtkTree()
		self.sw.add(self.root_tree)
		self.root_tree.show()

		self.browse = BrowseTreeItem(name, dict, self.disp)
		self.root_tree.append(self.browse)
		self.browse.init_subtree()
		self.browse.show()

class BrowseWindow(GtkWindow):
	def __init__(self, name, title):
		GtkWindow.__init__(self)
		self.set_title("Browse Window")

		box = GtkVBox()
		self.add(box)
		box.show()

		browse = BrowseVariables('Gtkinter', vars(Gtkinter))
		browse.border_width(10)
		box.pack_start(browse)
		browse.show()
		
		separator = GtkHSeparator()
		box.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("Close")
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
		self.close_button = button

if __name__ == '__main__':
	import Gtkinter
	win = BrowseWindow('Gtkinter', vars(Gtkinter))
	win.set_title("Browse Gtkinter")
	win.connect("destroy", mainquit)
	win.connect("delete_event", mainquit)
	win.close_button.connect("clicked", mainquit)
	win.show()

	mainloop()



#
# Constants for Gtkinter
#

#ANCHORED = 32768
ARROW_DOWN = 1
ARROW_LEFT = 2
ARROW_RIGHT = 3
ARROW_UP = 0
BASIC = 65536
#BEING_DESTROYED = 2
BUTTONBOX_DEFAULT = -1
BUTTONBOX_DEFAULT_STYLE = 0
BUTTONBOX_EDGE = 2
BUTTONBOX_END = 4
BUTTONBOX_SPREAD = 1
BUTTONBOX_START = 3
CAN_DEFAULT = 8192
CAN_FOCUS = 2048
CELL_EMPTY = 0
CELL_PIXMAP = 2
CELL_PIXTEXT = 3
CELL_TEXT = 1
CELL_WIDGET = 4
CENTIMETERS = 2
CURVE_TYPE_FREE = 2
CURVE_TYPE_LINEAR = 0
CURVE_TYPE_SPLINE = 1
DESTROYED = 1
DIRECTION_LEFT = 0
DIRECTION_RIGHT = 1
DIR_DOWN = 3
DIR_LEFT = 4
DIR_RIGHT = 5
DIR_TAB_BACKWARD = 1
DIR_TAB_FORWARD = 0
DIR_UP = 2
EXPAND = 1
FALSE = 0
FLOATING = 2
FILL = 4
HAS_DEFAULT = 16384
HAS_FOCUS = 4096
HAS_GRAB = 32768
#HAS_SHAPE_MASK = 4194304
INCHES = 1
#IN_CALL = 4
JUSTIFY_CENTER = 2
JUSTIFY_FILL = 3
JUSTIFY_LEFT = 0
JUSTIFY_RIGHT = 1
LEFT_RIGHT = 1
MAPPED = 128
MENU_FACTORY_MENU = 0
MENU_FACTORY_MENU_BAR = 1
MENU_FACTORY_OPTION_MENU = 2
#NEED_DESTROY = 1
NO_WINDOW = 32
ORIENTATION_HORIZONTAL = 0
ORIENTATION_VERTICAL = 1
PACK_END = 1
PACK_START = 0
PARENT_SENSITIVE = 1024
PIXELS = 0
POLICY_ALWAYS = 0
POLICY_AUTOMATIC = 1
POS_BOTTOM = 3
POS_LEFT = 0
POS_RIGHT = 1
POS_TOP = 2
PREVIEW_COLOR = 0
PREVIEW_GRAYSCALE = 1
PRIORITY_HIGH = -20
PRIORITY_INTERNAL = -10
PRIORITY_DEFAULT = 0
PRIORITY_LOW = 10
#PROPAGATE_STATE = 16384
RC_STYLE = 262144
REALIZED = 64
#REDRAW_PENDING = 524288
RESERVED_1 = 4
RESERVED_2 = 8
RESERVED_3 = 131072
#RESIZE_NEEDED = 2097152
#RESIZE_PENDING = 1048576
RUN_BOTH = 3
RUN_FIRST = 1
RUN_LAST = 2
RUN_MASK = 15
RUN_NO_RECURSE = 16
SCROLL_NONE = 0
SCROLL_PAGE_BACKWARD = 3
SCROLL_PAGE_FORWARD = 4
SCROLL_STEP_BACKWARD = 1
SCROLL_STEP_FORWARD = 2
SELECTION_BROWSE = 1
SELECTION_EXTENDED = 3
SELECTION_MULTIPLE = 2
SELECTION_SINGLE = 0
SENSITIVE = 512
SHADOW_ETCHED_IN = 3
SHADOW_ETCHED_OUT = 4
SHADOW_IN = 1
SHADOW_NONE = 0
SHADOW_OUT = 2
SHRINK = 2
STATE_ACTIVE = 1
STATE_INSENSITIVE = 4
STATE_NORMAL = 0
STATE_PRELIGHT = 2
STATE_SELECTED = 3
TOOLBAR_BOTH = 2
TOOLBAR_ICONS = 0
TOOLBAR_TEXT = 1
TOP_BOTTOM = 0
TOPLEVEL = 16
TREE_VIEW_ITEM = 1
TREE_VIEW_LINE = 0
TROUGH_END = 2
TROUGH_NONE = 0
TROUGH_START = 1
TRUE = 1
TYPE_ARGS = 16
TYPE_BOOL = 3
TYPE_BOXED = 13
TYPE_CALLBACK = 15
TYPE_CHAR = 2
TYPE_C_CALLBACK = 19
TYPE_DOUBLE = 9
TYPE_ENUM = 11
TYPE_FLAGS = 12
TYPE_FLOAT = 8
TYPE_FOREIGN = 14
TYPE_INT = 4
TYPE_INVALID = 0
TYPE_LONG = 6
TYPE_NONE = 1
TYPE_NUM_BUILTINS = 57
TYPE_OBJECT = 20
TYPE_POINTER = 17
TYPE_SIGNAL = 18
TYPE_STRING = 10
TYPE_UINT = 5
TYPE_ULONG = 7
#UNMAPPED = 32
UPDATE_ALWAYS = 1
UPDATE_CONTINUOUS = 0
UPDATE_DELAYED = 2
UPDATE_DISCONTINUOUS = 1
UPDATE_IF_VALID = 2
UPDATE_SNAP_TO_TICKS = 4
#USER_STYLE = 131072
VISIBILITY_NONE = 0
VISIBILITY_PARTIAL = 1
VISIBILITY_FULL = 2
VISIBLE = 256
WINDOW_DIALOG = 1
WINDOW_POPUP = 2
WINDOW_TOPLEVEL = 0
WIN_POS_CENTER = 1
WIN_POS_MOUSE = 2
WIN_POS_NONE = 0




#!/usr/bin/python

# translation of "Hello World III" from GTK manual, using new() function
# from Gtkinter.  Also implements a tooltip for the button.

from Gtkinter import *

def hello(*args):
	print "Hello World"
	window.destroy()

def destroy(*args):
	window.hide()
	mainquit()

tt = GtkTooltips()
tt.set_delay(500)

window = new(GtkWindow, {
	'GtkWindow::type':            WINDOW_TOPLEVEL,
	'GtkWindow::title':           'Hello World',
	'GtkWindow::allow_grow':      0,
	'GtkWindow::allow_shrink':    0,
	'GtkObject::signal::destroy': destroy,
	'GtkContainer::border_width': 10
})

button = new(GtkButton, {
	'GtkButton::label': 'Hello World',
	'GtkObject::signal::clicked': hello,
	'GtkWidget::parent': window,
	'GtkWidget::visible': 1
})

tt.set_tip(button, 'Prints "Hello World"', '')
tt.enable()

window.show()
mainloop()





#!/usr/bin/python

# this is a translation of "Hello World III" from the GTK manual,
# using Gtkinter

from Gtkinter import *

def hello(*args):
    print "Hello World"
    window.destroy()

def destroy(*args):
    window.hide()
    mainquit()

window = GtkWindow(WINDOW_TOPLEVEL)
window.connect("destroy", destroy)
window.border_width(10)

button = GtkButton("Hello World")
button.connect("clicked", hello)
window.add(button)
button.show()

window.show()

mainloop()





#!/usr/bin/python
#
# Tests of python-gtk.
#

import sys
from Gtkinter import *
import GtkExtra

class Application:		
	def __init__(self, argv):
		
		# hardcoded font - maybe not on every server ?
		self.medium_fonts= [
			"lucidasanstypewriter-10",
			"lucidasanstypewriter-12",
			"lucidasanstypewriter-14",
			"lucidasanstypewriter-18",
			"lucidasanstypewriter-24"
			]
		self.bold_fonts= [
			"lucidasanstypewriter-bold-10",
			"lucidasanstypewriter-bold-12",
			"lucidasanstypewriter-bold-14",
			"lucidasanstypewriter-bold-18",
			"lucidasanstypewriter-bold-24"
			]
		self.font_type=0	
		self.font_index=0
		# dictionary of loaded fonts 
		self.fonts={}
								
		self.w_window=GtkWindow()
		self.w_window.set_title("Test application")
		self.w_window.border_width(10)		
		self.w_vbox=GtkVBox()

		self.init_menu()
		self.init_text()
		
		self.w_window.add(self.w_vbox)
		self.w_vbox.show()
		self.w_window.show()		
		
	def mainloop(self):
		mainloop()

	def init_menu(self):
		self.w_menubar=GtkExtra.MenuFactory()
		
		self.w_menubar.add_entries([
                        ('File/New',        None, self.process_file_new),
                        ('File/Open...',"<control>O", self.process_file_open),
                        ('File/Save',       None, self.process_file_save),
                        ('File/Save As...', None, self.process_file_save_as),
                        ('File/Close',      None, self.process_file_close),
                        ('File/<separator>',None, None),
                        ('File/Exit',       None, self.process_file_exit),

                        ('Edit/Cut',   None, self.process_edit_cut),
                        ('Edit/Copy',  None, self.process_edit_copy),
                        ('Edit/Paste', None, self.process_edit_paste),

                        ('Help/About', None, self.process_help_about)
                ])
		self.w_window.add_accelerator_table(self.w_menubar.accelerator)
		self.w_vbox.pack_start(self.w_menubar, expand=FALSE)
		self.w_menubar.show()
		

	def init_text(self):		
		self.w_table=GtkTable(2,2,0)
		self.w_table.show()

		self.w_text=GtkText()
		self.w_text.set_usize(350,250)
		## Note: editable text is disable in gtk+-971109
		self.w_text.set_editable(0)
		self.w_table.attach(self.w_text,0,1,0,1,xpadding=1,ypadding=1)		
		self.w_text.show()

		self.w_text.connect("key_press_event", self.process_key)
		self.w_text.set_events(GDK.KEY_PRESS_MASK)

		self.w_hscrollbar=GtkHScrollbar(self.w_text.get_hadjustment())
		self.w_table.attach(self.w_hscrollbar,0,1,1,2,
				    yoptions=FILL)
		self.w_hscrollbar.show()
		
		self.w_vscrollbar=GtkVScrollbar(self.w_text.get_vadjustment())
		self.w_table.attach(self.w_vscrollbar,1,2,0,1,
				    xoptions=FILL)
		self.w_vscrollbar.show()
				
		self.w_vbox.pack_start(self.w_table)
					
	def process_file_new(self, mi):
		print "process_file_new"
		self.w_text.freeze()
		self.w_text.set_point(0)
		self.w_text.insert_defaults("*new file*")
		self.w_text.thaw()
		#self.w_text.queueDraw()
		
	def process_file_open(self, mi):
		print "process_file_open"
		print GtkExtra.file_open_box(modal=FALSE), "chosen"

	def process_file_save(self, mi):
		print "process_file_save"
		GtkExtra.message_box("Test Application", "Not Implemented",
				     ("OK",), pixmap='bomb.xpm')
				
	def process_file_save_as(self, mi):
		print "process_file_save_as"
		print GtkExtra.message_box(modal=FALSE), "chosen"

	def process_file_close(self, mi):
		print "process_file_close"
		GtkExtra.message_box("Test Application", "Not Implemented",
				     ("OK",), pixmap='bomb.xpm')

	def process_file_exit(self, mi):
		mainquit()
		
	def process_edit_cut(self, mi):
		print "process_edit_cut"
		
	def process_edit_copy(self, mi):
		print "process_edit_copy"
		
	def process_edit_paste(self, mi):
		print "process_edit_paste"

	def process_help_about(self, mi):
		GtkExtra.message_box("Test Application",
		"""Test Application for Python-Gtk.
		This is free software.
		This application tests key press events and text display.
		Any entered key (printable character) is displayed on
		screen. <F1> toggle medium and bold font. <F2> permits
		to change font size.
		""", ("OK",), pixmap='question.xpm')

	def process_key(self, win, event):
		if(event.keyval>=32 and event.keyval<=256):
			if(self.font_type==0):
				font=self.medium_fonts[self.font_index]
				if(not self.fonts.has_key(font)):
					self.fonts[font] = load_font(font)
				f=self.fonts[font]
			else:
				font=self.bold_fonts[self.font_index]
				if(not self.fonts.has_key(font)):
					self.fonts[font] = load_font(font)
				f=self.fonts[font]
			self.w_text.freeze()
			style = self.w_text.get_style()
			self.w_text.insert(f, style.fg[STATE_NORMAL],
					   style.bg[STATE_NORMAL],
					   chr(event.keyval))
			self.w_text.thaw()
		else:	
			# hardcoded key interpretation... XXX need some python keysymdef ?
			if(event.keyval==65470):
				# this is F1
				self.font_type = (self.font_type+1)%2
			elif(event.keyval==65471):	
				# this is F2
				self.font_index=(self.font_index+1)%len(self.medium_fonts)
		
if(__name__=="__main__"):
	use_defaults=0
	for arg in sys.argv:
		if(arg=="-d"):
			import pdb
			pdb.set_trace()

		if(arg=="-n"):
			use_defaults=0
			
	if(use_defaults==1):
		gtk.rc_parse("defaults.rc")
	
	app=Application(sys.argv)
	app.mainloop()
	



#!/usr/bin/python

import sys
import bdb
import repr
import string
import linecache # for linecache.getlines(filename)
from Gtkinter import *
import GtkExtra

class GtkDb(GtkWindow, bdb.Bdb):
	def __init__(self):
		GtkWindow.__init__(self)
		bdb.Bdb.__init__(self)
		self.realize()

		self.set_title("GtkDb")
		self.connect("destroy", self.do_quit)
		self.connect("delete_event", self.do_quit)

		self.box = GtkVBox()
		self.add(self.box)
		self.box.show()

		toolbar = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_BOTH)
		toolbar.set_space_size(10)
		self.box.pack_start(toolbar, expand=FALSE)
		toolbar.show()

		toolbar.append_item("next", "Next statement", None,
				    GtkPixmap(self, "next.xpm"), self.do_next)
		toolbar.append_item("step", "Step into function", None,
				    GtkPixmap(self, "step.xpm"), self.do_step)
		toolbar.append_space()
		toolbar.append_item("return",
				    "Continue execution to end of function",
				    None, GtkPixmap(self, "return.xpm"),
				    self.do_return)
		toolbar.append_space()
		toolbar.append_item("continue",
				    "Continue execution to next break point",
				    None, GtkPixmap(self, "continue.xpm"),
				    self.do_continue)
		toolbar.append_item("break",
				    "toggle break point at selected line",None,
				    GtkPixmap(self,"break.xpm"),self.do_break)
		toolbar.append_space()
		toolbar.append_item("edit",
				    "edit the value of the selected variable",
				    None, GtkPixmap(self, "edit.xpm"),
				    self.do_edit)
		toolbar.append_item("run",
			      "Execute some code in the current stack context",
			      None, GtkPixmap(self, "run.xpm"),
			      self.do_run)
		toolbar.append_space()
		toolbar.append_item("quit", "Quit the debugger", None,
				    GtkPixmap(self, "quit.xpm"), self.do_quit)

		sep = GtkHSeparator()
		self.box.pack_start(sep, expand=FALSE)
		sep.show()

		vpane = GtkVPaned()
		self.box.pack_start(vpane)
		vpane.show()

		hpane = GtkHPaned()
		vpane.add1(hpane)
		hpane.show()

		self.stackdisp = GtkCList(1, ['Stack Frame'])
		self.stackdisp.connect("select_row", self.update_curstack)
		self.stackdisp.set_usize(280, 125)
		self.stackdisp.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.stackdisp.set_selection_mode(SELECTION_BROWSE)
		self.stackdisp.border_width(2)
		hpane.add1(self.stackdisp)
		self.stackdisp.show()

		self.vardisp = GtkCList(3, ['local var', 'type', 'value'])
		self.vardisp.connect("select_row", self.update_selectedvar)
		self.vardisp.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.vardisp.set_selection_mode(SELECTION_BROWSE)
		self.vardisp.set_column_width(0, 75)
		self.vardisp.set_column_width(1, 30)
		self.vardisp.border_width(2)
		hpane.add2(self.vardisp)
		self.vardisp.show()
		self.vardisp.selected = 0
		self.vardisp.varnames = []

		self.filedisp = GtkCList(3, ['break', 'lineno', 'line'])
		self.minibreak = GtkPixmap(self, "minibreak.xpm")
		# This is giving a warning -- probably a bug in GTK
		self.filedisp.set_column_widget(0, self.minibreak)
		self.filedisp.column_titles_show()
		self.filedisp.set_column_width(0, 14)
		self.filedisp.set_column_width(1, 35)
		self.filedisp.connect("select_row", self.update_selection)
		self.filedisp.set_column_justification(1, JUSTIFY_RIGHT)
		self.filedisp.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.filedisp.set_selection_mode(SELECTION_BROWSE)
		self.filedisp.border_width(2)
		self.filedisp.set_usize(600, 200)
		vpane.add2(self.filedisp)
		self.filedisp.show()

		separator = GtkHSeparator()
		self.box.pack_start(separator, expand=FALSE)
		separator.show()

		align = GtkAlignment(0.0, 0.5, 0.0, 0.0)
		self.box.pack_start(align, expand=FALSE)
		align.show()
		self.status = GtkLabel()
		self.status.set_padding(4, 1)
		align.add(self.status)
		self.status.show()

		self.filename = None
		self.selected = 0
		self.blockupdate = 0

	def set_status(self, str):
		self.status.set(str)

	def update_selection(self, clist, r, c, event):
		if self.blockupdate: return
		self.selected = r + 1
		if c == 0: # break point column
			self.do_break()

	def update_curstack(self, clist, r, c, event):
		if self.blockupdate: return
		self.curindex = r
		self.curframe = self.stack[self.curindex][0]
		self.lineno = None
		self.update_code_listing()
		self.update_var_listing()

	def update_selectedvar(self, clist, r, c, event):
		self.vardisp.selected = r
		
	def set_quit(self):
		self.hide()
		self.destroy()
		bdb.Bdb.set_quit(self)

	def reset(self):
		bdb.Bdb.reset(self)
		self.forget()
	def forget(self):
		self.lineno = None
		self.stack = []
		self.curindex = 0
		self.curframe = None
	def setup(self, f, t):
		self.forget()
		self.stack, self.curindex = self.get_stack(f, t)
		self.curframe = self.stack[self.curindex][0]

	# interaction functions -- overriden from bdb
	def user_line(self, frame):
		# called when we stop or break at this line
		self.interaction(frame, None)
	def user_return(self, frame, return_value):
		# called when a return trap is set here
		frame.f_locals['__return__'] = return_value
		if frame.f_code.co_name:
			func = frame.f_code.co_name
		else:
			func = "<lambda>"
		self.set_status(func + " rerturned " + repr.repr(return_value))
		self.interaction(frame, None)
	def user_exception(self, frame, (exc_type, exc_value, exc_traceback)):
		frame.f_locals['__exception__'] = exc_type, exc_value
		if type(exc_type) == type(''):
			exc_type_name = exc_type
		else: exc_type_name = exc_type.__name__
		self.set_status(exc_type_name + ':' + repr.repr(exc_value))
		self.interaction(frame, exc_traceback)

	def interaction(self, frame, traceback):
		self.setup(frame, traceback)
		self.update_stack_listing(self.curindex)
		mainloop()
		self.forget()

	def update_stack_listing(self, curindex):
		self.stackdisp.freeze()
		self.blockupdate = 1
		self.stackdisp.clear()
		for i in range(len(self.stack)):
			frame_lineno = self.stack[i]
			row = self.format_stack_entry(frame_lineno, "##!##")
			row = string.split(row, "##!##")[0]
			self.stackdisp.append([row])
		self.blockupdate = 0
		self.stackdisp.moveto(curindex, 0, 1.0, 0.0)
		self.stackdisp.select_row(curindex, 0)
		self.stackdisp.thaw()

	def update_var_listing(self):
		self.vardisp.freeze()
		self.vardisp.clear()
		locals = self.curframe.f_locals
		self.vardisp.varnames = locals.keys()
		self.vardisp.varnames.sort()
		for var in self.vardisp.varnames:
			row = [var, type(locals[var]).__name__, repr.repr(
				locals[var])]
			self.vardisp.append(row)
		self.vardisp.thaw()

	def update_code_listing(self):
		frame = self.curframe
		newfile = frame.f_code.co_filename
		if newfile != self.filename:
			lines = linecache.getlines(newfile)
			self.filename = newfile
			self.filedisp.freeze()
			self.blockupdate = 1
			self.filedisp.clear()
			breaks = self.get_file_breaks(newfile)
			for line in range(len(lines)):
				self.filedisp.append(['', repr.repr(line+1),
						      lines[line]])
				if line+1 in breaks:
					self.filedisp.set_pixmap(line, 0,
							 self.minibreak)
			self.blockupdate = 0
			self.filedisp.thaw()
		self.selected = frame.f_lineno
		lineno = self.selected
		self.filedisp.moveto(lineno - 1, 2)
		self.filedisp.select_row(lineno - 1, 2)

	def do_next(self, _b=None):
		self.set_next(self.curframe)
		mainquit()
	def do_step(self, _b=None):
		self.set_step()
		mainquit()
	def do_return(self, _b=None):
		self.set_return(self.curframe)
		mainquit()
	def do_continue(self, _b=None):
		self.set_continue()
		mainquit()
	def do_quit(self, _b=None, _e=None):
		self.set_quit()
		mainquit()
	def do_break(self, _b=None):
		breaks = self.get_file_breaks(self.filename)
		if self.selected in breaks:
			err = self.clear_break(self.filename, self.selected)
			if err:
				self.set_status(err)
				return
			self.filedisp.set_text(self.selected-1, 0, ' ')
		else:
			err = self.set_break(self.filename, self.selected)
			if err:
				self.set_status(err)
				return
			self.filedisp.set_pixmap(self.selected-1, 0,
						 self.minibreak)
	def do_run(self, _b=None):
		line = GtkExtra.input_box("Execute Code",
					  "Enter code to execute:")
		if line == None: return
		locals = self.curframe.f_locals
		globals = self.curframe.f_globals
		globals['__privileged__'] = 1
		try:
			code = compile(line + '\n', '<stdin>', 'single')
			exec code in globals, locals
		except:
			if type(sys.exc_type) == type(''):
				exc_type_name = sys.exc_type
			else: exc_type_name = sys.exc_type.__name__
			self.set_status('*** ' + exc_type_name + ': ' +
					str(sys.exc_value))
			return
		self.update_var_listing()

	def do_edit(self, _b=None):
		locals = self.curframe.f_locals
		varname = self.vardisp.varnames[self.vardisp.selected]
		val = repr.repr(locals[varname])
		value = GtkExtra.input_box("Edit Variable",
				"Enter new value for" +	varname + ":", val)
		if value == None: return
		globals = self.curframe.f_globals
		globals['__privileged__'] = 1
		try:
			val = eval(value, globals, locals)
			self.curframe.f_locals[varname] = val
		except:
			if type(sys.exc_type) == type(''):
				exc_type_name = sys.exc_type
			else: exc_type_name = sys.exc_type.__name__
			self.set_status('*** ' + exc_type_name + ': ' +
					str(sys.exc_value))
			return
		row = self.vardisp.selected
		self.vardisp.set_text(row, 1, type(val).__name__)
		self.vardisp.set_text(row, 2, repr.repr(val))

# this makes up the interface that is compatible with pdb.
def run(statement, globals=None, locals=None):
	win = GtkDb()
	win.show()
	win.run(statement, globals, locals)

def runeval(expression, globals=None, locals=None):
	win = GtkDb()
	win.show()
	return win.runeval(expression, globals, locals)

def runcall(*args):
	win = GtkDb()
	win.show()
	return apply(win.runcall, args)

def set_trace():
	win = GtkDb()
	win.show()
	win.set_trace()

def post_mortem(traceback):
	win = GtkDb()
	win.show()
	win.reset()
	win.interaction(None, traceback)

def pm():
	post_mortem(sys.last_traceback)

if __name__ == '__main__':
	import os
	if not sys.argv[1:]:
		print "usage: gtkdb.py scriptfile [args ...]"
		sys.exit(2)
	filename = sys.argv[1]
	del sys.argv[0] # delete gtkdb.py
	sys.path.insert(0, os.path.dirname(filename))

	run('execfile("' + filename + '")', {'__name__': '__main__'})



#!/usr/bin/python

from Gtkinter import *
import GtkExtra
import gtkcons, gtkdb, gtkprof, edit
import os, sys, string

# select a good VT emulator
for vt in 'Eterm', 'nxterm', 'xterm-color', 'xterm', 'rxvt':
	for dirname in string.split(os.environ['PATH'], os.pathsep):
		fullname = os.path.join(dirname, vt)
		if os.path.exists(fullname):
			VT_CMD = fullname + ' -geometry 80x6 -e '
			break
	else:
		continue
	break
else:
	VT_CMD=''  # this is not ideal

class Application(GtkWindow):
	def __init__(self):
		GtkWindow.__init__(self, WINDOW_TOPLEVEL)
		self.connect("destroy", self.quit)
		self.connect("delete_event", self.quit)
		self.set_title("Python")
		self.set_usize(475, 325)
		self.main_box = GtkVBox()
		self.add(self.main_box)
		self.main_box.show()
		hdlbox = GtkHandleBox()
		self.main_box.pack_start(hdlbox, expand=FALSE)
		hdlbox.show()
		mf = GtkExtra.MenuFactory()
		mf.add_entries([
			('File/New', '<control>N', self.file_new),
			('File/Open...', '<control>O', self.file_open),
			('File/<separator>', None, None),
			('File/Exit', '<control>Q', self.file_exit),
			('Edit/Copy', '<control>C', self.edit_copy),
			('Edit/Paste', '<control>V', self.edit_paste),
			('Edit/Clear', None, self.edit_clear),
			('Python/Reload Module...', None, self.python_reload),
			('Python/<separator>', None, None),
			('Python/Run...', None, self.python_run),
			('Python/Debug...', None, self.python_debug),
			('Python/Profile...', None, self.python_prof),
			('Help/About...', None, self.help_about),
		])
		self.add_accelerator_table(mf.accelerator)
		hdlbox.add(mf)
		mf.show()
		self.mf = mf
		self.interp = gtkcons.Console(
				namespace={'__builtins__': __builtins__,
					   '__name__': '__main__',
					   '__doc__': None}, quit_cb=self.quit)
		self.main_box.pack_start(self.interp)
		self.interp.show()
		self.interp.init()
		self.editwins = []
	def quit(self, *args):
		for win in self.editwins:
			if win.chk_save(): return
			win.hide()
			win.destroy()
		mainquit()

	def reload_file(self, fname):
		if not os.path.isfile(fname):
			GtkExtra.message_box("File Not Found", fname +
					     " was not found.", ("OK",))
		dir = os.path.dirname(fname)
		base = os.path.basename(fname)
		if dir not in sys.path: sys.path.insert(0, dir)
		if   string.lower(base[-3:]) == '.py':  base = base[:-3]
		elif string.lower(base[-4:]) == '.pyc': base = base[:-4]
		if not sys.modules.has_key(base):
			self.interp.run('import ' + base)
		else:
			self.interp.run('import ' + base)
			self.interp.run('reload(' + base + ')')

	# execute a python script normally or with the debugger or profiler
	def run_script(self, fname):
		if not fname or os.path.exists(fname):
			GtkExtra.message_box("Run","Invalid filename",("OK",))
			return
		args = GtkExtra.input_box("Arguments",
				 "Enter any command line arguments")
		if args == None: return
		os.system(VT_CMD+'python "'+fname+'" ' + args + ' &')
	def debug_script(self, fname):
		if not fname or os.path.exists(fname):
			GtkExtra.message_box("Debug", "Invalid filename",
					     ("OK",))
			return
		args = GtkExtra.input_box("Arguments",
				 "Enter any command line arguments")
		if args == None: return
		os.system(VT_CMD+'python '+gtkdb.__file__+' "'+fname+'" ' +
			  args + ' &')
	def profile_script(self, fname):
		if not fname or not os.ppath.exists(fname):
			GtkExtra.message_box("Profile", "Invalid filename",
					     ("OK",))
			return
		args = GtkExtra.input_box("Arguments",
				 "Enter any command line arguments")
		if args == None: return
		os.system(VT_CMD+'python '+gtkprof.__file__+' "'+fname+'" ' +
			  args + ' &')

	def add_py_menu(self, ew):
		def run(b=None, ew=ew, app=self): app.run_script(ew.fname)
		def dbg(b=None, ew=ew, app=self): app.debug_script(ew.fname)
		def prf(b=None, ew=ew, app=self): app.profile_script(ew.fname)
		py_menu = GtkExtra.MenuFactory(MENU_FACTORY_MENU)
		py_menu.add_entries([
			('Run...',     None, run),
			('Debug...',   None, dbg),
			('Profile...', None, prf)
		])
		mi = GtkMenuItem('Python')
		ew.mf.insert(mi, 2)
		mi.show()
		mi.set_submenu(py_menu)
		ew.mf.hide()
		ew.mf.show()
		
	def file_new(self, mi=None):
		ew = edit.EditWindow(quit_cb=self.rem_editwin)
		self.editwins.append(ew)
		self.add_py_menu(ew)
		ew.show()
		ew.set_usize(0,0)
	def file_open(self, mi=None):
		fname = GtkExtra.file_open_box(modal=FALSE)
		if fname:
			ew = edit.EditWindow(quit_cb=self.rem_editwin)
			ew.load_file(fname)
			self.editwins.append(ew)
			self.add_py_menu(ew)
			ew.show()
			ew.set_usize(0,0)
	def rem_editwin(self, win=None, event=None):
		for i in range(len(self.editwins)):
			if self.editwins[i] == win:
				del self.editwins[i]
				break
	def file_exit(self, mi=None):
		self.quit()
	def edit_copy(self, mi=None):
		self.interp.text.copy_clipboard(0)
	def edit_paste(self, mi=None):
		self.interp.line.paste_clipboard(0)
	def edit_clear(self, mi=None):
		self.interp.line.delete_selection()
	def python_reload(self, mi=None):
		print "python_reload"
	def python_run(self, mi=None):
		fname = GtkExtra.file_sel_box("Run")
		if fname:
			self.run_script(fname)
	def python_debug(self, mi=None):
		fname = GtkExtra.file_sel_box("Debug")
		if fname:
			self.debug_script(fname)
	def python_prof(self, mi=None):
		fname = GtkExtra.file_sel_box("Profile")
		if fname:
			self.profile_script(fname)
	def help_about(self, mi=None):
		print "help_about"

if __name__ == '__main__':
	app = Application()
	app.show()
	app.set_usize(0,0)
	mainloop()



#!/usr/bin/python

from Gtkinter import *
import GtkExtra

wins = {}

def delete_event(win, event=None):
	win.hide()
	# don't destroy window -- just leave it hidden
	return TRUE

def create_buttons(_button):
	if not wins.has_key("buttons"):
		win = GtkWindow()
		wins["buttons"] = win
		win.connect("delete_event", delete_event)
		win.set_title("buttons")
		box1 = GtkVBox()
		win.add(box1)
		box1.show()
		table = GtkTable(3, 3, FALSE)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		table.border_width(10)
		box1.pack_start(table)
		table.show()

		def toggle_show(b):
			if b.flags(VISIBLE):
				b.hide()
			else:
				b.show()
		button = map(lambda i: GtkButton("button"+str(i)), range(9))
		for i in range(9):
			button[i].connect("clicked",
					  lambda _b, _b2=button[(i+1)%9],
					  t=toggle_show: t(_b2))
			y,x = divmod(i, 3)
			table.attach(button[i], x,x+1, y,y+1)
			button[i].show()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["buttons"].show()
				   
def create_toggle_buttons(_button):
	if not wins.has_key("toggle_buttons"):
		win = GtkWindow()
		wins["toggle_buttons"] = win
		win.connect("delete_event", delete_event)
		win.set_title("toggle butttons")
		win.border_width(0)
		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()
		for i in range(1, 4):
			button = GtkToggleButton("button" + str(i))
			box2.pack_start(button)
			button.show()
		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["toggle_buttons"].show()

def create_check_buttons(_button):
	if not wins.has_key("check_buttons"):
		win = GtkWindow()
		wins["check_buttons"] = win
		win.connect("delete_event", delete_event)
		win.set_title("check buttons")
		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		for i in range(1,4):
			button = GtkCheckButton("button" + str(i))
			box2.pack_start(button)
			button.show()
		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["check_buttons"].show()

def create_radio_buttons(_button):
	if not wins.has_key("radio_buttons"):
		win = GtkWindow()
		wins["radio_buttons"] = win
		win.connect("delete_event", delete_event)
		win.set_title("radio buttons")
		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

                button1 = GtkRadioButton(None, "button1")
		box2.pack_start(button1)
		button1.show()
		for i in range(2,4):
			button = GtkRadioButton(button1, "button" + str(i))
			box2.pack_start(button)
			button.show()
		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["radio_buttons"].show()

def create_button_box(_button):
	if not wins.has_key("button_box"):
		win = GtkDialog()
		win.set_title("button box")
		wins["button_box"] = win
		win.connect("delete_event", delete_event)

		box1 = GtkHButtonBox()
		box1.set_usize(550, -1)
		box1.set_spacing(25)
		box1.set_layout(BUTTONBOX_EDGE)
		box1.border_width(10)
		win.vbox.pack_start(box1)
		box1.show()

		def bbox_start(_b, bbox=box1):
			bbox.set_layout(BUTTONBOX_START)
			bbox.queue_resize()
		def bbox_edge(_b, bbox=box1):
			bbox.set_layout(BUTTONBOX_EDGE)
			bbox.queue_resize()
		def bbox_spread(_b, bbox=box1):
			bbox.set_layout(BUTTONBOX_SPREAD)
			bbox.queue_resize()
		def bbox_end(_b, bbox=box1):
			bbox.set_layout(BUTTONBOX_END)
			bbox.queue_resize()

		button = GtkButton("start")
		button.connect("clicked", bbox_start)
		box1.add(button)
		button.show()
		button = GtkButton("edge")
		button.connect("clicked", bbox_edge)
		box1.add(button)
		button.show()
		button = GtkButton("spread")
		button.connect("clicked", bbox_spread)
		box1.add(button)
		button.show()
		button = GtkButton("end")
		button.connect("clicked", bbox_end)
		box1.add(button)
		button.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		win.action_area.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["button_box"].show()

def make_toolbar(win):
		toolbar = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_BOTH)
		bg = win.get_style().bg[STATE_NORMAL]
		if not win.flags(REALIZED):
			win.realize()

		def set_toolbar_horizontal(_b, toolbar=toolbar):
			toolbar.set_orientation(ORIENTATION_HORIZONTAL)
		def set_toolbar_vertical(_b, toolbar=toolbar):
			toolbar.set_orientation(ORIENTATION_VERTICAL)
		def set_toolbar_icons(_b, toolbar=toolbar):
			toolbar.set_style(TOOLBAR_ICONS)
		def set_toolbar_text(_b, toolbar=toolbar):
			toolbar.set_style(TOOLBAR_TEXT)
		def set_toolbar_both(_b, toolbar=toolbar):
			toolbar.set_style(TOOLBAR_BOTH)
		def set_toolbar_small_space(_b, toolbar=toolbar):
			toolbar.set_space_size(5)
		def set_toolbar_big_space(_b, toolbar=toolbar):
			toolbar.set_space_size(10)
		def set_toolbar_enable(_b, toolbar=toolbar):
			toolbar.set_tooltips(TRUE)
		def set_toolbar_disable(_b, toolbar=toolbar):
			toolbar.set_tooltips(FALSE)
			print "Hello"

		toolbar.append_item("Horizontal", "Horizontal toolbar layout",
				    "Horizontal toolbar layout",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_horizontal)
		toolbar.append_item("Vertical", "Vertical toolbar layout",
				    "Vertical toolbar layout",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_vertical)
		toolbar.append_space()
		toolbar.append_item("Icons", "Only show toolbar icons",
				    "Only Icons",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_icons)
		toolbar.append_item("Text", "Only show toolbar text",
				    "Only Text",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_text)
		toolbar.append_item("Both", "Show toolbar icons and text",
				    "Both Text and Icons",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_both)
		toolbar.append_space()

		entry = GtkEntry()
		entry.show()
		toolbar.append_widget(entry, "", "")

		toolbar.append_space()
		toolbar.append_item("Small", "Use small spaces",
				    "Small spaces",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_small_space)
		toolbar.append_item("Big", "Use big spaces",
				    "Big spaces",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_big_space)
		toolbar.append_space()
		toolbar.append_item("Enable", "Enable tooltips",
				    "Enable tooltips",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_enable)
		toolbar.append_item("Disable", "Disable tooltips",
				    "Disable tooltips",
				    GtkPixmap(win, "test.xpm", bg),
				    set_toolbar_disable)
		return toolbar

def create_toolbar(_button):
	if not wins.has_key("toolbar"):
		win = GtkWindow()
		wins["toolbar"] = win
		win.set_title("Toolbar test")
		win.set_policy(FALSE, TRUE, TRUE)
		win.connect("delete_event", delete_event)
		win.realize()
		toolbar = make_toolbar(win)
		win.add(toolbar)
		toolbar.show()
	wins["toolbar"].show()

def create_handle_box(_button):
	if not wins.has_key("handle_box"):
		win = GtkWindow()
		wins["handle_box"] = win
		win.set_title("Handle Box Test")
		win.connect("delete_event", delete_event)
		win.border_width(20)
		win.realize()

		hbox = GtkHandleBox()
		win.add(hbox)
		hbox.show()

		toolbar = make_toolbar(win)
		hbox.add(toolbar)
		toolbar.show()
	wins["handle_box"].show()

def create_reparent(_button):
	if not wins.has_key("reparent"):
		win = GtkWindow()
		wins["reparent"] = win
		win.connect("delete_event", delete_event)
		win.set_title("buttons")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkHBox(spacing=5)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		label = GtkLabel("Hello World")
		frame = GtkFrame("Frame 1")
		box2.pack_start(frame)
		frame.show()
		box3 = GtkVBox(spacing=5)
		box3.border_width(5)
		frame.add(box3)
		box3.show()
		def reparent_label(button, new_parent=box3, label=label):
			label.reparent(new_parent)
		button = GtkButton("switch")
		button.connect("clicked", reparent_label)
		box3.pack_start(button, expand=FALSE)
		button.show()

		box3.pack_start(label, expand=FALSE)
		label.show()

		frame = GtkFrame("Frame 2")
		box2.pack_start(frame)
		frame.show()
		box3 = GtkVBox(spacing=5)
		box3.border_width(5)
		frame.add(box3)
		box3.show()
		def reparent_label(button, new_parent=box3, label=label):
			label.reparent(new_parent)
		button = GtkButton("switch")
		button.connect("clicked", reparent_label)
		box3.pack_start(button, expand=FALSE)
		button.show()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["reparent"].show()

def create_pixmap(_button):
	if not wins.has_key("pixmap"):
		win = GtkWindow()
		wins["pixmap"] = win
		win.connect("delete_event", delete_event)
		win.set_title("pixmap")
		win.realize()

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		button = GtkButton()
		box2.pack_start(button, expand=FALSE, fill=FALSE)
		button.show()

		pixmap = GtkPixmap(win, "test.xpm",
				   button.get_style().bg[STATE_NORMAL])
		label = GtkLabel("Pixmap\ntest")
		box3 = GtkHBox()
		box3.border_width(2)
		box3.add(pixmap)
		box3.add(label)
		button.add(box3)
		pixmap.show()
		label.show()
		box3.show()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()
		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["pixmap"].show()

def create_tooltips(_button):
	if not wins.has_key("tooltips"):
		win = GtkWindow()
		wins["tooltips"] = win
		win.connect("delete_event", delete_event)
		win.set_title("tooltips")

		tooltips = GtkTooltips()

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		button = GtkToggleButton("button1")
		box2.pack_start(button)
		button.show()
		tooltips.set_tip(button, "This is button 1", "button 1")

		button = GtkToggleButton("button2")
		box2.pack_start(button)
		button.show()
		tooltips.set_tip(button, "This is button 2", "button 2")

		button = GtkToggleButton("button3")
		box2.pack_start(button)
		button.show()
		tooltips.set_tip(button, "This is button 3.  This is also a really long tooltip which probably won't fit on a single line and will therefore need to be wrapped.  Hopefully the wrapping will work correctly.", "long")

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()

		tooltips.set_tip(button, "Push this button to close window",
				 "push")
		tooltips.enable()
	wins["tooltips"].show()

def create_menu(depth):
	if depth < 1: return None
	menu = GtkMenu()
	group = None
	for i in range(5):
		menuitem = GtkRadioMenuItem(group,
					    "item %2d - %d" % (depth, i+1))
		group = menuitem
		menu.append(menuitem)
		menuitem.show()
		if depth > 1:
		        submenu = create_menu(depth - 1)
			menuitem.set_submenu(submenu)
	return menu
def create_menus(_button):
	if not wins.has_key("menus"):
		win = GtkWindow()
		wins["menus"] = win
		win.connect("delete_event", delete_event)
		win.set_title("menus")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		menubar = GtkMenuBar()
		box1.pack_start(menubar, expand=FALSE)
		menubar.show()

		menuitem = GtkMenuItem("test\nline2")
		menuitem.set_submenu(create_menu(2))
		menubar.append(menuitem)
		menuitem.show()

		menuitem = GtkMenuItem("foo")
		menuitem.set_submenu(create_menu(2))
		menubar.append(menuitem)
		menuitem.show()

		menuitem = GtkMenuItem("bar")
		menuitem.set_submenu(create_menu(2))
		menuitem.right_justify()
		menubar.append(menuitem)
		menuitem.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		optionmenu = GtkOptionMenu()
		optionmenu.set_menu(create_menu(1))
		box2.pack_start(optionmenu)
		optionmenu.show()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["menus"].show()

def create_scrolled_windows(_button):
	if not wins.has_key("scrolled_windows"):
		win = GtkDialog()
		wins["scrolled_windows"] = win
		win.connect("delete_event", delete_event)
		win.set_title("dialog")

		scrolled_window = GtkScrolledWindow()
		scrolled_window.border_width(10)
		scrolled_window.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		win.vbox.pack_start(scrolled_window)
		scrolled_window.show()

		table = GtkTable(20, 20)
		table.set_row_spacings(10)
		table.set_col_spacings(10)
		scrolled_window.add(table)
		table.show()

		for i in range(20):
			for j in range(20):
				button = GtkToggleButton("button (%d,%d)" %
							 (i, j))
				table.attach(button, i,i+1, j,j+1)
				button.show()
		button = GtkButton("close")
		button.connect("clicked", win.hide)
		win.action_area.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["scrolled_windows"].show()

def create_entry(_button):
	if not wins.has_key("entry"):
		win = GtkWindow()
		wins["entry"] = win
		win.connect("delete_event", delete_event)
		win.set_title("entry")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		entry = GtkEntry()
		entry.set_text("Hello World")
		box2.pack_start(entry)
		entry.show()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["entry"].show()

def create_list(_button):
	if not wins.has_key("list"):
		win = GtkWindow()
		wins["list"] = win
		win.connect("delete_event", delete_event)
		win.set_title("list")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		scrolled_win = GtkScrolledWindow()
		scrolled_win.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		box2.pack_start(scrolled_win)
		scrolled_win.show()

		list = GtkList()
		list.set_selection_mode(SELECTION_MULTIPLE)
		list.set_selection_mode(SELECTION_BROWSE)
		scrolled_win.add(list)
		list.show()

		list_items = ("hello", "world", "blah", "foo", "bar", "argh")
		for i in list_items:
			list_item = GtkListItem(i)
			list.add(list_item)
			list_item.show()

		def list_add(_button, list=list, num=[1]):
			list_item = GtkListItem("added item "+str(num[0]))
			num[0] = num[0] + 1
			list_item.show()
			list.add(list_item)

		def list_remove(_button, list=list):
			selection = list.get_selection()
			list.remove_items(selection)
			for i in selection: i.destroy()

		box3 = GtkHBox(spacing=10)
		box2.pack_start(box3, expand=FALSE)
		box3.show()

		button = GtkButton("add")
		button.connect("clicked", list_add)
		box3.pack_start(button)
		button.show()

		button = GtkButton("remove")
		button.connect("clicked", list_remove)
		box3.pack_start(button)
		button.show()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["list"].show()

def create_clist(_button):
	if not wins.has_key("clist"):
		win = GtkWindow()
		wins["clist"] = win
		win.connect("delete_event", delete_event)
		win.set_title("clist")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkHBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE, fill=FALSE)
		box2.show()

		clist = GtkCList(7, map(lambda i: "Title "+str(i), range(7)))
		clist.set_column_width(0, 100)
		for i in range(1, 7):
			clist.set_column_width(i, 80)

		text = map(lambda i: "Column "+str(i), range(7))
		text[1] = "Right"
		text[2] = "Center"

		def add1000_clist(_button, clist=clist, t=text):
			clist.freeze()
			for i in range(1000):
				t[0] = "Row "+str(i)
				clist.append(t)
			clist.thaw()
		def add10000_clist(_button, clist=clist, t=text):
			clist.freeze()
			for i in range(10000):
				t[0] = "Row "+str(i)
				clist.append(t)
			clist.thaw()
		def clear_clist(_button, clist=clist):
			clist.clear()
		s = [0]
		def remove_row_clist(_button, clist=clist, selected=s):
			clist.remove(selected[0])
		def select_clist(_clist, r, c, event, selected=s):
			selected[0] = r

		button = GtkButton("Add 1,000 Rows")
		box2.pack_start(button)
		button.connect("clicked", add1000_clist)
		button.show()

		button = GtkButton("Add 10,000 Rows")
		box2.pack_start(button)
		button.connect("clicked", add10000_clist)
		button.show()

		button = GtkButton("Clear List")
		box2.pack_start(button)
		button.connect("clicked", clear_clist)
		button.show()

		button = GtkButton("Remove Row")
		box2.pack_start(button)
		button.connect("clicked", remove_row_clist)
		button.show()

		box2 = GtkHBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE, fill=FALSE)
		box2.show()

		button = GtkButton("Show Title Buttons")
		box2.pack_start(button)
		button.connect("clicked", clist.column_titles_show)
		button.show()

		button = GtkButton("Hide Title Buttons")
		box2.pack_start(button)
		button.connect("clicked", clist.column_titles_hide)
		button.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		clist.set_row_height(20)
		clist.connect("select_row", select_clist)
		clist.set_column_width(0, 100)

		for i in range(1, 7):
			clist.set_column_width(0, 80)
		clist.set_selection_mode(SELECTION_BROWSE)
		clist.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		clist.set_column_justification(1, JUSTIFY_RIGHT)
		clist.set_column_justification(2, JUSTIFY_CENTER)

		for i in range(100):
			text[0] = "Row "+str(i)
			clist.append(text)
		clist.border_width(5)
		box2.pack_start(clist)
		clist.show()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["clist"].show()

def create_color_selection(_button):
	if not wins.has_key("color_selection"):
		win = GtkColorSelectionDialog("color selection dialog")
		wins["color_selection"] = win
		win.colorsel.set_opacity(TRUE)
		win.colorsel.set_update_policy(UPDATE_CONTINUOUS)
		win.position(WIN_POS_MOUSE)
		win.connect("delete_event", delete_event)

		win.cancel_button.connect("clicked", win.hide)
	wins["color_selection"].show()

def create_file_selection(_button):
	if not wins.has_key("file_selection"):
		win = GtkFileSelection("file selection dialog")
		wins["file_selection"] = win
		win.connect("delete_event", delete_event)
		def file_selection_ok(_button, fs=win):
			print fs.get_filename()
		win.ok_button.connect("clicked", file_selection_ok)
		win.cancel_button.connect("clicked", win.hide)
	wins["file_selection"].show()

def create_dialog(_button):
	if not wins.has_key("dialog"):
		win = GtkDialog()
		wins["dialog"] = win
		win.connect("delete_event", delete_event)
		win.set_title("dialog")

		button = GtkButton("OK")
		button.connect("clicked", win.hide)
		win.action_area.pack_start(button)
		button.show()

		def label_toggle(_button, label=[None], box=win.vbox):
			l = label[0]
			if not l:
				l = GtkLabel("Dialog Test")
				l.set_padding(10, 10)
				box.pack_start(l)
				l.show()
				label[0] = l
			else:
				l.destroy()
				label[0] = None

		button = GtkButton("Toggle")
		button.connect("clicked", label_toggle)
		win.action_area.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["dialog"].show()

def create_range_controls(_button):
	if not wins.has_key("range_controls"):
		win = GtkWindow()
		wins["range_controls"] = win
		win.connect("delete_event", delete_event)
		win.set_title("range_controls")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		adjustment = GtkAdjustment(0, 0, 101, 0.1, 1, 1)

		scale = GtkHScale(adjustment)
		scale.set_usize(150, 30)
		scale.set_update_policy(UPDATE_DELAYED)
		scale.set_digits(1)
		scale.set_draw_value(TRUE)
		box2.pack_start(scale)
		scale.show()

		scrollbar = GtkHScrollbar(adjustment)
		scrollbar.set_update_policy(UPDATE_DELAYED)
		box2.pack_start(scrollbar)
		scrollbar.show()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["range_controls"].show()

def create_rulers(_button):
	if not wins.has_key("rulers"):
		win = GtkWindow()
		wins["rulers"] = win
		win.connect("delete_event", delete_event)
		win.set_title("rulers")
		win.set_usize(300, 300)
		# these should be defined elsewhere
		GDK_POINTER_MOTION_MASK      = 1 << 2
		GDK_POINTER_MOTION_HINT_MASK = 1 << 3
		win.set_events(GDK_POINTER_MOTION_MASK |
			       GDK_POINTER_MOTION_HINT_MASK)

		table = GtkTable(2, 3)
		win.add(table)
		table.show()

		ruler = GtkHRuler()
		ruler.set_range(5, 15, 0, 20)
		ruler.set_metric(PIXELS)
		# XXX decide how to connect motion event
		table.attach(ruler, 1,2, 0,1, yoptions=FILL)
		ruler.show()

		ruler = GtkVRuler()
		ruler.set_range(5, 15, 0, 20)
		ruler.set_metric(PIXELS)
		# XXX notify ,,,
		table.attach(ruler, 0,1, 1,2, xoptions=FILL)
		ruler.show()

		# remove when fixed
		label = GtkLabel("Sometime I'll really\n" +
				 "have to fix these\n" +
				 "rulers.")
		table.attach(label, 1,2, 1,2)
		label.show()

		# XXX Finish this
	wins["rulers"].show()

def create_text(_button):
	if not wins.has_key("text"):
		win = GtkWindow()
		wins["text"] = win
		win.connect("delete_event", delete_event)
		win.set_title("text")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		table = GtkTable(2, 2)
		table.set_row_spacing(0, 2)
		table.set_col_spacing(0, 2)
		box2.pack_start(table)
		table.show()

		text = GtkText()
		text.set_editable(FALSE)
		table.attach(text, 0,1, 0,1)
		text.show()

		hscrollbar = GtkHScrollbar(text.get_hadjustment())
		table.attach(hscrollbar, 0,1, 1,2, yoptions=FILL)
		hscrollbar.show()

		vscrollbar = GtkVScrollbar(text.get_vadjustment())
		table.attach(vscrollbar, 1,2, 0,1, xoptions=FILL)
		vscrollbar.show()

		text.freeze()
		text.realize()
#		style = text.get_style()
#		font = style.font ; fg = style.white; bg = style.bg[
	        text.insert_defaults("some text\n")
		text.insert_defaults("more text\n")
		text.insert_defaults("\n")
		text.insert_defaults("(a blank line)\n")
		text.insert_defaults("\n")
		text.insert_defaults("beaverloop\n")
		text.insert_defaults("\n")
		text.insert_defaults("Bad Acid ... Jazz!\n")
		text.insert_defaults("\n")
		text.insert_defaults("They all call me mellow yellow,\n")
		text.insert_defaults("Thats 'cause I'm a mellow fellow,\n")
		text.insert_defaults("You walk by and I'll say hello,\n")
		text.thaw()

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["text"].show()

def create_notebook(_button):
	if not wins.has_key("notebook"):
		win = GtkWindow()
		wins["notebook"] = win
		win.connect("delete_event", delete_event)
		win.set_title("notebook")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		notebook = GtkNotebook()
		notebook.set_tab_pos(POS_TOP)
		box2.pack_start(notebook)
		notebook.show()

		for i in range(5):
			frame = GtkFrame("Page " + str(i))
			frame.border_width(10)
			frame.set_usize(200, 150)
			frame.show()
			label = GtkLabel("Page " + str(i))
			frame.add(label)
			label.show()
			label = GtkLabel("Tab " + str(i))
			notebook.append_page(frame, label)
		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkHBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.show()

		button = GtkButton("next")
		button.connect("clicked", notebook.next_page)
		box2.pack_start(button)
		button.show()

		button = GtkButton("prev")
		button.connect("clicked", notebook.prev_page)
		box2.pack_start(button)
		button.show()

		def rotate_notebook(_b, note=notebook, pos=[POS_TOP]):
			pos[0] = (pos[0] + 1) % 4
			note.set_tab_pos(pos[0])

		button = GtkButton("rotate")
		button.connect("clicked", rotate_notebook)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["notebook"].show()


def create_panes(_button):
	if not wins.has_key("panes"):
		win = GtkWindow()
		wins["panes"] = win
		win.connect("delete_event", delete_event)
		win.set_title("Panes")

		vpaned = GtkVPaned()
		win.add(vpaned)
		vpaned.border_width(5)
		vpaned.show()

		hpaned = GtkHPaned()
		vpaned.add1(hpaned)

		frame = GtkFrame()
		frame.set_shadow_type(SHADOW_IN)
		frame.set_usize(60, 60)
		hpaned.add1(frame)
		frame.show()

		frame = GtkFrame()
		frame.set_shadow_type(SHADOW_IN)
		frame.set_usize(80, 60)
		hpaned.add2(frame)
		frame.show()

		hpaned.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		vpaned.add2(box2)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins["panes"].show()

def create_dnd(_button):
	if not wins.has_key("dnd"):
		# note that these two functions don't use any global variables
		# to communicate.  In fact, try openning two copies of
		# testgtk.py (or the C version) and drag between them.
		def dnd_drop(button, event):
			msg = "Drop data of type %s was:\n\n%s" % \
				(event.data_type, event.data)
			GtkExtra.message_box("Drop", msg,
				("Continue with life in\n" +
				"spite of this oppression",))
		def dnd_drag_request(button, event):
			dnd_string = "Bill Gates demands royalties for\n" + \
				     "your use of his innovation."
			button.dnd_data_set(event, dnd_string)
		win = GtkWindow(WINDOW_TOPLEVEL)
		wins["dnd"] = win
		win.connect("delete_event", lambda win,_e: win.hide())
		win.set_title("Drag -N- Drop")

		box1 = GtkVBox()
		win.add(box1)
		box1.show()

		box2 = GtkHBox(spacing=5)
		box2.border_width(10)
		box1.pack_start(box2)
		box2.show()

		frame = GtkFrame("Drag")
		box2.pack_start(frame)
		frame.show()

		box3 = GtkVBox(spacing=5)
		box3.border_width(5)
		frame.add(box3)
		box3.show()

		button = GtkButton("Drag me!")
		box3.pack_start(button)
		button.show()
		button.realize()
		button.connect("drag_request_event", dnd_drag_request)
		button.dnd_drag_set(TRUE, ['text/plain'])

		frame = GtkFrame("Drop")
		box2.pack_start(frame)
		frame.show()

		box3 = GtkVBox(spacing=5)
		box3.border_width(5)
		frame.add(box3)
		box3.show()

		button = GtkButton("To")
		box3.pack_start(button)
		button.show()
		button.realize()
		button.connect("drop_data_available_event", dnd_drop)
		button.dnd_drop_set(TRUE, ['text/plain'], FALSE)

		separator = GtkHSeparator()
		box1.pack_start(separator, expand=FALSE)
		separator.show()

		box2 = GtkVBox(spacing=10)
		box2.border_width(10)
		box1.pack_start(box2, expand=FALSE)
		box2.show()

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		box2.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	wins['dnd'].show()

def create_progress_bar(_button):
	if not wins.has_key("progress_bar"):
		win = GtkDialog()
		wins["progress_bar"] = win
		win.set_title("dialog")

		vbox = GtkVBox(spacing=5)
		vbox.border_width(10)
		win.vbox.pack_start(vbox)
		vbox.show()

		label = GtkLabel("progress...")
		label.set_alignment(0, 0.5)
		vbox.pack_start(label, expand=FALSE)
		label.show()

		pbar = GtkProgressBar()
		pbar.set_usize(200, 20)
		vbox.pack_start(pbar)
		pbar.show()

		def timeout(pbar=pbar, pcnt=[0]):
			pcnt[0] = (pcnt[0] + 0.02) % 1.0
			pbar.update(pcnt[0])
			return TRUE

		win.func = [timeout, 0]

		def killit(arg1=None, arg2=None, tid=win.func, win=win):
			timeout_remove(tid[1])
			win.hide()
		win.connect("destroy", killit)
		win.connect("delete_event", killit)

		button = GtkButton("close")
		button.connect("clicked", killit)
		win.action_area.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()
	win = wins["progress_bar"]
	win.show()
	win.func[1] = timeout_add(100, win.func[0])

def create_gamma_curve(_button):
	if not wins.has_key("gamma_curve"):
		win = GtkWindow()
		wins["gamma_curve"] = win
		win.connect("delete_event", lambda _w, _e, win=win: win.hide())
		win.set_title("test")
		win.set_usize(200, 150)

		curve = GtkGammaCurve()
		win.add(curve)
		curve.realize()
		curve.show()
	wins["gamma_curve"].show()

def create_timeout_test(_button):
	if not wins.has_key("timeout_test"):
		win = GtkDialog()
		wins["timeout_test"] = win
		win.connect("delete_event", delete_event)
		win.set_title("Timeout Test")

		label = GtkLabel("count: 0")
		label.set_padding(10, 10)
		win.vbox.pack_start(label)
		label.show()

		func = [None, 0]
		def timeout_test(label=label, count=[0]):
			count[0] = count[0] + 1
			label.set("count: " + str(count[0]))
			return TRUE

		func = [timeout_test, 0]
		def start_timeout_test(_button, func=func):
			if not func[1]:
				func[1] = timeout_add(100, func[0])
		def stop_timeout_test(_button, func=func):
			if func[1]:
				timeout_remove(func[1])
				func[1] = 0

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		win.action_area.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()

		button = GtkButton("start")
		button.connect("clicked", start_timeout_test)
		win.action_area.pack_start(button)
		button.show()

		button = GtkButton("stop")
		button.connect("clicked", stop_timeout_test)
		win.action_area.pack_start(button)
		button.show()
	wins["timeout_test"].show()

def create_idle_test(_button):
	if not wins.has_key("idle_test"):
		win = GtkDialog()
		wins["idle_test"] = win
		win.connect("delete_event", delete_event)
		win.set_title("Idle Test")

		label = GtkLabel("count: 0")
		label.set_padding(10, 10)
		win.vbox.pack_start(label)
		label.show()

		def idle_test(label=label, count=[0]):
			count[0] = count[0] + 1
			label.set("count: " + str(count[0]))
			return TRUE

		win.func = [idle_test, 0]
		def start_idle_test(_button, func=win.func):
			if not func[1]:
				func[1] = idle_add(func[0])
		def stop_idle_test(_button, func=win.func):
			if func[1]:
				idle_remove(func[1])
				func[1] = 0

		button = GtkButton("close")
		button.connect("clicked", win.hide)
		win.action_area.pack_start(button)
		button.set_flags(CAN_DEFAULT)
		button.grab_default()
		button.show()

		button = GtkButton("start")
		button.connect("clicked", start_idle_test)
		win.action_area.pack_start(button)
		button.show()

		button = GtkButton("stop")
		button.connect("clicked", stop_idle_test)
		win.action_area.pack_start(button)
		button.show()
	wins["idle_test"].show()

def do_exit(button):
	mainquit()

def create_main_window():
	buttons = {
		"buttons": create_buttons,
		"toggle buttons": create_toggle_buttons,
		"check buttons": create_check_buttons,
		"radio buttons": create_radio_buttons,
		"button box": create_button_box,
		"toolbar": create_toolbar,
		"handle box": create_handle_box,
		"reparent": create_reparent,
		"pixmap": create_pixmap,
		"tooltips": create_tooltips,
		"menus": create_menus,
		"scrolled windows": create_scrolled_windows,
		"drawing areas": None,
		"entry": create_entry,
		"list": create_list,
		"clist": create_clist,
		"color selection": create_color_selection,
		"file selection": create_file_selection,
		"dialog": create_dialog,
		"miscellaneous": None,
		"range controls": create_range_controls,
		"rulers": create_rulers,
		"text": create_text,
		"notebook": create_notebook,
		"panes": create_panes,
		"shapes": None, #create_shapes,
		"dnd": create_dnd,
		"progress bar": create_progress_bar,
		"preview color": None, #create_color_preview,
		"preview gray": None, #create_gray_preview,
		"gamma curve": create_gamma_curve,
		"test scrolling": None, #creaate_scroll_text,
		"test selection": None, #create_selection_test,
		"test timeout": create_timeout_test,
		"test idle": create_idle_test,
		"test": None, #create_test
	}
	win = GtkWindow()
	win.set_name("main window")
	win.set_usize(200, 400)
	win.set_uposition(20, 20)
	win.connect("destroy", mainquit)
	win.connect("delete_event", mainquit)
	win.set_title("gtktest")
	box1 = GtkVBox(FALSE, 0)
	win.add(box1)
	box1.show()
	scrolled_window = GtkScrolledWindow()
	scrolled_window.border_width(10)
	scrolled_window.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
	box1.pack_start(scrolled_window)
	scrolled_window.show()
	box2 = GtkVBox()
	box2.border_width(10)
	scrolled_window.add(box2)
	box2.show()
	k = buttons.keys()
	k.sort()
	for i in k:
		button = GtkButton(i)
		if buttons[i]:
			button.connect("clicked", buttons[i])
		else:
			button.set_sensitive(FALSE)
		box2.pack_start(button)
		button.show()
	separator = GtkHSeparator()
	box1.pack_start(separator, expand=FALSE)
	separator.show()
	box2 = GtkVBox(spacing=10)
	box2.border_width(10)
	box1.pack_start(box2, expand=FALSE)
	box2.show()
	button = GtkButton("close")
	button.connect("clicked", do_exit)
	button.set_flags(CAN_DEFAULT)
	box2.pack_start(button)
	button.grab_default()
	button.show()
	win.show()

def main():
        rc_parse("testgtkrc")
	create_main_window()
	mainloop()

if __name__ == '__main__': main()



#!/usr/bin/python

import string

objects = ( "Adjustment", "Alignment", "Arrow", "AspectFrame", "Bin", "Box",
  "Button",
  "ButtonBox", "CheckButton", "CheckMenuItem", "CList", "ColorSelection",
  "ColorSelectionDialog", "Combo", "Container", "Curve", "Data", "Dialog",
  "DrawingArea", "Editable", "Entry", "EventBox", "FileSelection", "Fixed",
  "Frame",
  "GammaCurve", "HandleBox", "HBox", "HButtonBox", "HPaned", "HRuler",
  "HScale", "HScrollbar", "HSeparator", "Image", "InputDialog", "Item",
  "Label", "List", "ListItem", "Menu", "MenuBar", "MenuItem", "MenuShell",
  "Misc", "Notebook", "Object", "OptionMenu", "Paned", "Pixmap", "Preview",
  "ProgressBar", "RadioButton", "RadioMenuItem", "Range", "Ruler", "Scale",
  "Scrollbar", "ScrolledWindow", "Separator", "SpinButton", "Statusbar",
  "Table", "Text", "TipsQuery",
  "ToggleButton", "Toolbar", "Tooltips", "Tree", "TreeItem", "VBox",
  "VButtonBox",
  "Viewport", "VPaned", "VRuler", "VScale", "VScrollbar", "VSeparator",
  "Widget", "Window" )

f = open("obj_typemap.i", "w")

f.write("# typemaps for Gtk objects to Python equivalents\n\n");

for obj in objects:
	OBJ = ''
	for c in obj:
		if c in string.uppercase:
			OBJ = OBJ + '_' + c
		else:
			OBJ = OBJ + string.upper(c)
	if obj == 'CList': OBJ = '_CLIST'
	f.write("%%typemap(python,in) Gtk%s * {\n" % (obj,))
	f.write("        if (PyGtk_Check($source))\n")
	f.write("            $target = GTK%s(PyGtk_Get($source));\n" % (OBJ,))
	f.write("        else {\n")
	f.write('            PyErr_SetString(PyExc_TypeError, "object not a Gtk Object");\n')
	f.write("            return NULL;\n")
	f.write("        }\n")
	f.write("    }\n\n")
	f.write("%%typemap(python,out) Gtk%s * {\n" % (obj,))
	f.write("        $target = PyGtk_New((GtkObject *)$source);\n")
	f.write("    }\n\n")

f.close()




#!/usr/bin/python

# This is a sample implementation of an editor.

from Gtkinter import *
import GtkExtra
import os

BLOCK_SIZE = 2048

class EditWindow(GtkWindow):
	def __init__(self, quit_cb=None):
		GtkWindow.__init__(self, WINDOW_TOPLEVEL)
		self.set_usize(470, 300)
		self.connect("delete_event", self.file_exit)

		self.quit_cb = quit_cb

		self.table = GtkTable(2,3)
		self.add(self.table)
		self.table.show()
		hdlbox = GtkHandleBox()
		self.table.attach(hdlbox, 0,2, 0,1, xoptions=FILL,
				  yoptions=FILL)
		hdlbox.show()
		self.menubar = self.create_menu()
		hdlbox.add(self.menubar)
		self.menubar.show()

		self.text = GtkText()
		self.text.connect("changed", self.set_dirty)
		self.text.set_editable(TRUE)
		self.table.attach(self.text, 0,1, 1,2)
		self.text.show()

		vadj = self.text.get_vadjustment()
		self.vscroll = GtkVScrollbar(vadj)
		self.table.attach(self.vscroll, 1,2, 1,2, xoptions=FILL)
		self.vscroll.show()
		vadj.connect('changed', self.chk_scroll)

		self.text.realize()
		self.dirty = 0
		self.file_new()
		self.text.grab_focus()

	def set_dirty(self, text=None):
		self.dirty = 1
	def chk_scroll(self, adj):
		if adj.upper - adj.lower <= adj.page_size:
			self.vscroll.hide()
		else:
			self.vscroll.show()

	def load_file(self, fname):
		try:
			fd = open(fname)
			self.text.freeze()
			self.text.delete_text(0, self.text.get_length())
			buf = fd.read(BLOCK_SIZE)
			while buf != '':
				self.text.insert_defaults(buf)
				buf = fd.read(BLOCK_SIZE)
			self.text.thaw()
			self.text.get_vadjustment().set_value(0)
			self.text.queue_draw()
			self.set_title(os.path.basename(fname))
			self.fname = fname
			self.dirty = 0
			self.new = 0
		except:
			GtkExtra.message_box('Edit', "Can't open " + fname,
					     ("OK",))

	def create_menu(self):
		mf = GtkExtra.MenuFactory()

		mf.add_entries([
			('File/New',        '<control>N', self.file_new),
			('File/Open...',    '<control>O', self.file_open),
			('File/Save',       '<control>S', self.file_save),
			('File/Save As...', None,         self.file_saveas),
			('File/<separator>',None,         None),
			('File/Exit',       '<control>Q', self.file_exit),
			('Edit/Cut',        '<control>X', self.edit_cut),
			('Edit/Copy',       '<control>C', self.edit_copy),
			('Edit/Paste',      '<control>V', self.edit_paste),
			('Edit/Clear',      None,         self.edit_clear),
			('Edit/<separator>',None,         None),
			('Edit/Find...',    None,         self.edit_find),
			('Edit/Find Next',  None,         self.edit_find_next),
			('Help/About...',   None,         self.help_about)
		])
		# activate key bindings ...
		self.add_accelerator_table(mf.accelerator)
		self.mf = mf
		return mf

	def chk_save(self):
		if self.dirty:
			ret = GtkExtra.message_box("Unsaved File",
						   (self.fname or "Untitled")+
						   " has not been saved\n" +
						   "Do you want to save it?",
						   ("Yes", "No", "Cancel"))
			if ret == None or ret == "Cancel": return 1
			if ret == "Yes": self.file_save()
		return 0

	def file_new(self, mi=None):
		if self.chk_save(): return
		self.text.freeze()
		self.text.delete_text(0, self.text.get_length())
		self.text.thaw()
		self.dirty = 0
		self.fname = None
		self.set_title("Untitled")
		self.new = 1
	def file_open(self, mi=None):
		if self.chk_save(): return
		fname = GtkExtra.file_open_box(modal=FALSE)
		if not fname: return
		self.load_file(fname)
	def file_save(self, mi=None):
		if self.new:
			self.file_saveas()
			return
		try:
			pos = 0
			length = self.text.get_length()
			fd = open(self.fname, "w")
			while pos < length:
				buf = self.text.get_chars(pos,
						  min(pos+BLOCK_SIZE, length))
				if buf != None: fd.write(buf)
				pos = pos + BLOCK_SIZE
			self.dirty = 0
		except:
			GtkExtra.message_box("Save", "Error saving file " +
					     self.fname, ("OK",))
		
	def file_saveas(self, mi=None):
		fname = GtkExtra.file_save_box(modal=FALSE)
		if not fname: return
		self.fname = fname
		self.set_title(os.path.basename(fname))
		self.new = 0
		self.file_save()
	def file_exit(self, mi=None, event=None):
		if self.chk_save(): return
		self.hide()
		self.destroy()
		if self.quit_cb: self.quit_cb(self)
	def edit_cut(self, mi):
		self.text.cut_clipboard(0)
	def edit_copy(self, mi):
		self.text.copy_clipboard(0)
	def edit_paste(self, mi):
		self.text.paste_clipboard(0)
	def edit_clear(self, mi):
		self.text.delete_selection()
	# I'll implement these later
	def edit_find(self, mi): pass
	def edit_find_next(self, mi): pass
	def help_about(self, mi):
		GtkExtra.message_box("Edit Window", "Copyright (C) 1998  " +
				     "James Henstridge\n" +
				     "This program is covered by the GPL>=2",
				     ("OK",))

def edit(fname, mainwin=FALSE):
	if mainwin: quit_cb = mainquit
	else:       quit_cb = None
	w = EditWindow(quit_cb=quit_cb)
	w.load_file(fname)
	w.show()
	w.set_usize(0,0)
	if mainwin: mainloop()
if __name__ == '__main__':
	import sys
	edit(sys.argv[-1], mainwin=TRUE)



#!/usr/bin/python

# this is a translation of "Hello World III" from the GTK manual,
# using gtkmodule

from gtk import *
from GTK import *

def hello(*args):
    print "Hello World"
    gtk_widget_destroy(window)

def destroy(*args):
    gtk_widget_hide(window)
    gtk_main_quit()

gtk_init()

window = gtk_window_new(WINDOW_TOPLEVEL)
gtk_signal_connect(window, "destroy", destroy)
gtk_container_border_width(window, 10)

button = gtk_button_new_with_label("Hello World")
gtk_signal_connect(button, "clicked", hello)
gtk_container_add(window, button)
gtk_widget_show(button)

gtk_widget_show(window)

gtk_main()





#
# GdkImlib: An interface to gdk_imlib to go with Gtkinter.  It wraps the
#           native module _gdkimlib.  It is sugested that you import it as
#           "import GdkImlib".
#
# Author: James Henstridge <james@daa.com.au>
#
# To use GdkImlib, create an instance of the GdkImlib.Image class.  The
# constructor takes a filename as its one argument.  Then call the render
# method (optionally with width and height arguments), and finally call
# the make_pixamp method to get a GTK pixmap widget with the image loaded.
#
# For extra information, read the rest of this file, and Imlib's documentation.
#

import _gdkimlib

from Gtkinter import _obj2inst

#some constants ...
RT_PLAIN_PALETTE       = 0
RT_PLAIN_PALETTE_FAST  = 1
RT_DITHER_PALETTE      = 2
RT_DITHER_PALETTE_FAST = 3
RT_PLAIN_TRUECOL       = 4
RT_DITHER_TRUECOL      = 5

_gdkimlib.gdk_imlib_init()

class Image:
	def __init__(self, filename="", _obj=None):
		if _obj: self._im = _obj; return
		self._im = _gdkimlib.gdk_imlib_load_image(filename)
	def __getattr__(self, attr):
		attrs = {
			'rgb_width':  _gdkimlib.gdk_imlib_get_rgb_width,
			'rgb_height': _gdkimlib.gdk_imlib_get_rgb_height,
			'filename':   _gdkimlib.gdk_imlib_get_filename
		}
		if attrs.has_key(attr):
			return attrs[attr](self._im)
		raise AttributeError, attr
	def render(self, width=None, height=None):
		return _gdkimlib.gdk_imlib_render(self._im,
						  width or self.rgb_width,
						  height or self.rgb_height)
	def get_image_border(self):
		return _gdkimlib.gdk_imlib_get_image_border(self._im)
	def set_image_border(self, left=0, right=0, top=0, bottom=0):
		_gdkimlib.gdk_imlib_set_image_border(self._im, (left, right,
								top, bottom))
	def get_image_shape(self):
		return _gdkimlib.gdk_imlib_get_image_shape(self._im)
	def set_image_shape(self, red, green, blue):
		_gdkimlib.gdk_imlib_set_image_shape(self._im,(red,green,blue))
	def save_image_to_eim(self, file):
		_gdkimlib.gdk_imlib_save_image_to_eim(self._im, file)
	def add_image_to_eim(self, file):
		_gdkimlib.gdk_imlib_add_image_to_eim(self._im, file)
	def save_image_to_ppm(self, file):
		_gdkimlib.gdk_imlib_save_image_to_ppm(self._im, file)
	def set_image_modifier(self, gamma, brightness, contrast):
		_gdkimlib.gdk_imlib_set_image_modifier(self._im,
					       (gamma, brightness, contrast))
	def set_image_red_modifier(self, gamma, brightness, contrast):
		_gdkimlib.gdk_imlib_set_image_red_modifier(self._im,
					       (gamma, brightness, contrast))
	def set_image_blue_modifier(self, gamma, brightness, contrast):
		_gdkimlib.gdk_imlib_set_image_red_modifier(self._im,
					       (gamma, brightness, contrast))
	def set_image_green_modifier(self, gamma, brightness, contrast):
		_gdkimlib.gdk_imlib_set_image_red_modifier(self._im,
					       (gamma, brightness, contrast))
	def get_image_modifier(self):
		return _gdkimlib.gdk_imlib_get_image_modifier(self._im)
	def get_image_red_modifier(self):
		return _gdkimlib.gdk_imlib_get_image_red_modifier(self._im)
	def get_image_green_modifier(self):
		return _gdkimlib.gdk_imlib_get_image_green_modifier(self._im)
	def get_image_blue_modifier(self):
		return _gdkimlib.gdk_imlib_get_image_blue_modifier(self._im)
	def set_image_red_curve(self, curve):
		_gdkimlib.gdk_imlib_set_image_red_curve(self._im, curve)
	def set_image_green_curve(self, curve):
		_gdkimlib.gdk_imlib_set_image_green_curve(self._im, curve)
	def set_image_blue_curve(self, curve):
		_gdkimlib.gdk_imlib_set_image_blue_curve(self._im, curve)
	def get_image_red_curve(self):
		return _gdkimlib.gdk_imlib_get_image_red_curve(self._im)
	def get_image_green_curve(self):
		return _gdkimlib.gdk_imlib_get_image_green_curve(self._im)
	def get_image_blue_curve(self):
		return _gdkimlib.gdk_imlib_get_image_blue_curve(self._im)
	def apply_modifiers_to_rgb(self):
		_gdkimlib.gdk_imlib_apply_modifiers_to_rgb(self._im)
	def changed_image(self):
		_gdkimlib.gdk_imlib_changed_image(self._im)
	def flip_image_horizontal(self):
		_gdkimlib.gdk_imlib_flip_image_horizontal(self._im)
	def flip_image_vertical(self):
		_gdkimlib.gdk_imlib_flip_image_vertical(self._im)
	def rotate_image(self, d):
		_gdkimlib.gdk_imlib_rotate_image(self._im, d)
	def clone_image(self):
		return Image(_obj=_gdkimlib.gdk_imlib_clone_image(self._im))
	def clone_scaled_image(self, width, height):
		return Image(_obj=_gdkimlib.gdk_imlib_clone_scaled_image(
			self._im, width, height))
	def make_pixmap(self):
		return _obj2inst(_gdkimlib.gdk_imlib_make_pixmap(self._im))

def get_render_type():
	return _gdkimlib.gdk_imlib_get_render_type()
def set_render_type(rend_type):
	_gdkimlib.gdk_imlib_set_render_type(rend_type)
def load_colors(file):
	return _gdkimlib.gdk_imlib_load_colors(file)
def free_colors():
	_gdkimlib.gdk_imlib_free_colors()
def get_fallback():
	return _gdkimlib.gdk_imlib_get_fallback()
def set_fallback(fallback):
	_gdkimlib.gdk_imlib_set_fallback(fallback)
def get_sysconfig():
	return _gdkimlib.gdk_imlib_get_sysconfig()
def best_color_match(r, g, b):
	return _gdkimlib.gdk_imlib_best_color_match(r, g, b)



#!/usr/bin/python
#
# Tests of pygtk
#

import sys
from Gtkinter import *
import GtkExtra
				
class Application:		
	def __init__(self, argv):
		self.w_window=GtkWindow(title="Test Application")
		self.w_window.border_width(10)		
		self.w_vbox=GtkVBox()

		self.init_menu()
		self.init_text()
		
		self.w_window.add(self.w_vbox)
		self.w_vbox.show()
		self.w_window.show()		
		
	def mainloop(self):
		mainloop()

	def init_menu(self):
		self.w_menubar=GtkExtra.MenuFactory()
                self.w_menubar.add_entries([
                        ('File/New',        None, self.process_file_new),
                        ('File/Open...','<control>O', self.process_file_open),
                        ('File/Save',       None, self.process_file_save),
                        ('File/Save As...', None, self.process_file_save_as),
                        ('File/Close',      None, self.process_file_close),
                        ('File/<separator>',None, None),
                        ('File/Exit',       None, self.process_file_exit),

                        ('Edit/Cut',   None, self.process_edit_cut),
                        ('Edit/Copy',  None, self.process_edit_copy),
                        ('Edit/Paste', None, self.process_edit_paste),
		])
		self.w_window.add_accelerator_table(self.w_menubar.accelerator)
		self.w_vbox.pack_start(self.w_menubar, expand=FALSE)
		self.w_menubar.show()
		

	def init_text(self):		
		self.w_table=GtkTable(2,2,0)
		self.w_table.show()
		
		self.w_text=GtkText()
		self.w_text.set_usize(350,250)
		## Note: editable text is disable in gtk+-971109
		self.w_text.set_editable(1)
		self.w_table.attach(self.w_text,0,1,0,1,xpadding=1,ypadding=1)
		self.w_text.show()

		self.w_hscrollbar=GtkHScrollbar(self.w_text.get_hadjustment())
		self.w_table.attach(self.w_hscrollbar,0,1,1,2,yoptions=FILL)
		self.w_hscrollbar.show()
		
		self.w_vscrollbar=GtkVScrollbar(self.w_text.get_vadjustment())
		self.w_table.attach(self.w_vscrollbar,1,2,0,1,xoptions=FILL)
		self.w_vscrollbar.show()
				
		self.w_vbox.pack_start(self.w_table)
					
	def process_file_new(self, mi):
		print "process_file_new"
		self.w_text.freeze()
		self.w_text.set_point(0)
		self.w_text.insert_defaults("*new file*")
		self.w_text.thaw()
		#self.w_text.queueDraw()
		
	def process_file_open(self, mi):
		print "process_file_open"
		fname = GtkExtra.file_open_box(modal=FALSE)
		if fname:
			try:
				f=open(fname, "r")
			except IOError:
				return
			self.w_text.freeze()
			while TRUE:
				line = f.readline()
				if line == "":
					break
				self.w_text.insert_defaults(line)
			self.w_text.thaw()
		
	def process_file_save(self, mi):
		print "process_file_save"
		GtkExtra.message_box("Test Application", "Not implemented",
				     pixmap='bomb.xpm')
				
	def process_file_save_as(self, mi):
		print "process_file_save_as"
		print GtkExtra.file_save_box(modal=FALSE), "chosen"

	def process_file_close(self, mi):
		print "process_file_close"

	def process_file_exit(self, mi):
		mainquit()
		
	def process_edit_cut(self, mi):
		print "process_edit_cut"
		self.w_text.cut_clipboard(0)
		
	def process_edit_copy(self, mi):
		print "process_edit_copy"
		self.w_text.copy_clipboard(0)
		
	def process_edit_paste(self, mi):
		print "process_edit_paste"
		self.w_text.paste_clipboard(0)

	def process_character(self, *args):
		print "process_character"
	def process_button(self, *args):
		print "process_button"
			
if(__name__=="__main__"):
	use_defaults=1
	for arg in sys.argv:
		if(arg=="-d"):
			import pdb
			pdb.set_trace()

		if(arg=="-n"):
			use_defaults=0
			
#	if(use_defaults==1):
#		Gtkrc_parse("defaults.rc")
	
	app=Application(sys.argv)
	app.mainloop()
	
